﻿using System;
using System.Data;
using System.Drawing;
using System.Net;
using System.Threading;
using System.Windows.Forms;
using Telerik.WinControls.UI;

namespace RadGridViewExample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            DataTable table = new DataTable();
            table.Columns.Add("Images");
            table.Rows.Add(@"http://fr.motorstv.com/sites/default/files/images/galerie/mini-wrc-3.jpg");
            table.Rows.Add(@"http://blogautomobile.fr/wp-content/uploads/2010/09/Mini-WRC_2011_04.jpg");
            table.Rows.Add(@"http://testdrive.bg/media/articles/Image/news/Mini/2011-countryman-wrc/Mini-WRC_2011_1024x768_wallpaper_01.jpg");
            table.Rows.Add(@"http://www.pistonspy.com/wp-content/uploads/2010/09/Mini-WRC-002-550x326.jpg");
            table.Rows.Add(@"http://1.bp.blogspot.com/-uvrSH59ChD8/Tm0CIruBc5I/AAAAAAAABxM/Jpbm7OlSWPI/s1600/2011-Mini-WRC-Side-View.jpg");
            table.Rows.Add(@"http://www.blogcdn.com/www.autoblog.com/media/2011/04/web630-mini-wrc-2.jpg");
            table.Rows.Add(@"http://www.creativecrash.com/system/photos/000/118/501/118501/big/mini_wrc_2011_5.jpg?1292662773");
            table.Rows.Add(@"http://www.auto-power-girl.com/high-resolution-wallpapers/mini-countryman-wrc/mini-countryman-wrc-2010-7.jpg");

            this.radGridView1.CreateCell += new GridViewCreateCellEventHandler(radGridView1_CreateCell);
            this.radGridView1.AllowAddNewRow = false;
            this.radGridView1.DataSource = table;
            this.radGridView1.Columns[0].Width = 160;
            this.radGridView1.TableElement.RowHeight = 100;
        }

        private void radGridView1_CreateCell(object sender, GridViewCreateCellEventArgs e)
        {
            if (e.CellType == typeof(GridDataCellElement))
            {
                e.CellElement = new CustomCellElement(e.Column, e.Row);
            }
        }
    }

    public class CustomCellElement : GridDataCellElement
    {
        public CustomCellElement(GridViewColumn column, GridRowElement row)
            : base(column, row)
        {
            this.ImageLayout = ImageLayout.Stretch;
        }

        public override void SetContent()
        {
            base.SetContent();

            this.Image = null;
            this.Text = "Loading image...";

            if (this.RowInfo.Cells[this.ColumnIndex].Value != null)
            {
                if (this.RowInfo.Cells[this.ColumnIndex].Tag != null &&
                    ((ImageInfo)this.RowInfo.Cells[this.ColumnIndex].Tag).Url == this.RowInfo.Cells[this.ColumnIndex].Value.ToString())
                {
                    this.Image = ((ImageInfo)this.RowInfo.Cells[this.ColumnIndex].Tag).Image;
                }
                else
                {
                    ThreadPool.QueueUserWorkItem(new WaitCallback(LoadImage));
                }
            }
        }

        private void LoadImage(object state)
        {
            string url = this.RowInfo.Cells[this.ColumnIndex].Value.ToString();
            HttpWebRequest request = (HttpWebRequest)System.Net.HttpWebRequest.Create(url);
            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            Image image = Image.FromStream(response.GetResponseStream());
            response.Close();

            this.Image = image;
            this.Text = url;

            ImageInfo info = new ImageInfo(url, image);
            this.RowInfo.Cells[this.ColumnIndex].Tag = info;
        }
    }

    public class ImageInfo
    {
        private string url;
        private Image image;

        public ImageInfo(string url, Image image)
        {
            this.url = url;
            this.image = image;
        }

        public string Url
        {
            get { return this.url; }
            set { this.url = value; }
        }

        public Image Image
        {
            get { return this.image; }
            set { this.image = value; }
        }
    }
}
